import { FileTypeResult } from 'file-type';

/**
 * @deprecated use DeployedMode instead
 */
declare enum HostedMode {
    NORMAL = 1,
    /**
     * 悬浮窗
     */
    OVERLAP = 16,
    WALLPAPER = 256,
    SCREEN = 4096,
    /**
     * 后台组件，没有界面，一般在后台执行定时任务
     */
    BACKGROUND = 65536,
    /**
     * 页面
     */
    PAGE = 1048576,
    ALL = 1118481
}
declare enum DeployMode {
    NORMAL = 1,
    /**
     * 悬浮窗
     */
    OVERLAP = 16,
    BACKGROUND = 65536,
    ALL = 1118481
}

interface Language {
    locale: string;
    language: string;
    code: string;
}
interface ILanguageMap {
    'ar-SA': Language;
    'bn-BD': Language;
    'en-US': Language;
    'de-DE': Language;
    'el-GR': Language;
    'es-ES': Language;
    'fi-FI': Language;
    'fil-PH': Language;
    'fr-FR': Language;
    'hi-IN': Language;
    'hu-HU': Language;
    'id-ID': Language;
    'it-IT': Language;
    'ja-JP': Language;
    'ko-KR': Language;
    'nl-NL': Language;
    'pl-PL': Language;
    'pt-BR': Language;
    'pt-PT': Language;
    'ro-RO': Language;
    'ru-RU': Language;
    'sv-SE': Language;
    'ta-IN': Language;
    'tr-TR': Language;
    'uk-UA': Language;
    'vi-VN': Language;
    'zh-CN': Language;
    'zh-TW': Language;
}
type LanguageCode = keyof ILanguageMap;
type LanguageTextMap = {
    [key in LanguageCode]?: string;
};

type WidgetPermission = 'keyboard' | 'mouse' | 'clipboard' | 'notification' | 'storage' | 'network' | 'cpu' | 'system-info';

type MetaInfo = {
    [key: string]: string | number | boolean;
};
interface IWindowSize {
    width: number;
    height: number;
    maxWidth?: number;
    maxHeight?: number;
    minWidth?: number;
    minHeight?: number;
}
interface IPageOptions extends IWindowSize {
    name: string;
    /**
     * 当showInSearch为true时,此项必填
     */
    title: LanguageTextMap;
    /**
     * 当showInSearch为true时,此项必填
     */
    description?: LanguageTextMap;
    /**
     * 当showInSearch为true时,此项必填
     */
    keywords?: WidgetKeyword[];
    lang: LanguageCode;
    packageName?: string;
    /**
     * 当showInSearch为true时,此项必填
     */
    previewImage?: string;
    /**
     * 是否在搜索中显示
     */
    showInSearch?: boolean;
    /**
     * 悬浮窗模式 是否可移动
     */
    movable?: boolean;
    resizable?: boolean;
    backgroundThrottling?: boolean;
    security?: boolean;
    path: string;
    /**
     * 是否只能添加一次
     */
    singleton?: boolean;
    permissions?: WidgetPermission[];
    meta?: MetaInfo;
    broadcastChannels?: string[];
}
interface IPage {
    readonly name: string;
    /**
     * 窗口标题，显示在界面上的，
     * https://zh.m.wikipedia.org/zh-hans/ISO_639-1
     */
    readonly title: LanguageTextMap;
    readonly permissions: WidgetPermission[];
    /**
     * 窗口介绍
     */
    readonly description: LanguageTextMap;
    readonly keywords: WidgetKeyword[];
    /**
     * 组件默认语言
     */
    readonly lang: LanguageCode;
    readonly width: number;
    packageName?: string | null;
    readonly height: number;
    readonly maxWidth: number;
    readonly maxHeight: number;
    readonly minWidth: number;
    readonly minHeight: number;
    readonly movable: boolean;
    readonly singleton: boolean;
    readonly resizable: boolean;
    readonly path: string;
    readonly meta: {
        [key: string]: string | number | boolean;
    };
    readonly backgroundThrottling: boolean;
    /**
     * 预览图片，可以是GIF
     */
    readonly previewImage?: string;
}
declare class Page implements IPage {
    readonly name: string;
    /**
     * 窗口标题，显示在界面上的，
     * https://zh.m.wikipedia.org/zh-hans/ISO_639-1
     */
    readonly title: LanguageTextMap;
    /**
     * 窗口介绍
     */
    readonly description: LanguageTextMap;
    readonly keywords: WidgetKeyword[];
    readonly security: boolean;
    readonly permissions: WidgetPermission[];
    /**
     * 组件默认语言
     */
    readonly lang: LanguageCode;
    readonly width: number;
    packageName?: string | null;
    readonly height: number;
    readonly maxWidth: number;
    readonly maxHeight: number;
    readonly minWidth: number;
    readonly minHeight: number;
    readonly movable: boolean;
    readonly singleton: boolean;
    readonly resizable: boolean;
    readonly path: string;
    readonly meta: {
        [key: string]: string | number | boolean;
    };
    readonly backgroundThrottling: boolean;
    /**
     * 预览图片，可以是GIF
     */
    readonly previewImage?: string;
    constructor(options: IPageOptions);
    /**
     * 获取组件标题
     * @param lang 语言环境，不传则获取默认语言
     */
    getTitle(lang?: LanguageCode): string | undefined;
    /**
     * 获取组件描述
     * @param lang 语言环境，不传则获取默认标题
     */
    getDescription(lang?: LanguageCode): string | undefined;
    static parseJSON(json: string): Page;
    static parseObject(obj: any): Page;
    isResizable(): boolean;
}

interface IWidgetOptions extends IWindowSize, IPageOptions {
    supportDeployMode?: DeployMode;
    configPagePath?: string;
    previewImage: string;
    description: LanguageTextMap;
    keywords: WidgetKeyword[];
    /**
     */
    routes?: WidgetRoute[];
}
/**
 * @deprecated
 */
interface WidgetRoute {
    url: string;
    name: string;
}
declare class Widget extends Page {
    readonly previewImage: string;
    readonly supportDeployMode: number;
    /**
     * 配置页面路径，没有则不能修改
     */
    readonly configPagePath?: string;
    /**
     * @deprecated
     */
    readonly routes: WidgetRoute[];
    constructor(options: IWidgetOptions);
    static parseJSON(json: string): Widget;
    static parseObject(obj: any): Widget;
    /**
     * 是否支持悬浮窗
     */
    isSupportOverlap(): boolean;
    isSupportBackground(): boolean;
    /**
     * 是否支持普通模式
     */
    isSupportNormal(): boolean;
    isConfigurable(): boolean;
}
declare enum WidgetKeyword {
    RECOMMEND = "recommend",
    TOOLS = "tools",
    EFFICIENCY = "efficiency",
    PICTURE = "picture",
    LIFE = "life",
    SHORTCUT = "shortcut",
    COUNTDOWN = "countdown",
    TIMER = "timer",
    INFO = "info",
    DASHBOARD = "dashboard"
}

interface LocalPackageUrlInfo {
    /**
     * 入口文件，默认为index.html，也可以是/
     * @example remote package: https://rtugeek.gitee.io/hotspot/index.html
     * @example local package: index.html
     */
    entry?: string;
    /**
     * 是否使用hash路由
     */
    hash: boolean;
}
interface RemotePackageUrlInfo extends LocalPackageUrlInfo {
    /**
     * 部署到服务器的base path，只在远程组件包中生效，需以/开始
     * @example /hotspot
     */
    base?: string;
}
interface WidgetPackageOptions extends LocalPackageUrlInfo {
    name: string;
    version?: string;
    author: string;
    homepage?: string;
    title: LanguageTextMap;
    permissions?: WidgetPermission[];
    description: LanguageTextMap;
    remoteEntry?: string;
    remotePackage?: string;
    remote?: RemotePackageUrlInfo;
    icon?: string;
    lang?: LanguageCode;
    widgets?: Widget[];
    devOptions?: DevOptions;
}
declare class WidgetPackage implements RemotePackageUrlInfo {
    /**
     * 组件包名,一般为域名倒写，e.g. com.example
     */
    readonly name: string;
    /**
     * 组件包版本，可以为空，默认采用package.json里的版本
     * e.g. 1.0.2
     */
    version?: string;
    /**
     * 组件作者署名
     */
    readonly author: string;
    /**
     * 组件首页
     */
    readonly homepage?: string;
    /**
     * 组件描述
     */
    readonly title: {
        [key: LanguageCode | string]: string;
    };
    /**
     * 组件描述
     */
    readonly description: {
        [key: LanguageCode | string]: string;
    };
    /**
     * 组件入口文件，通常为 index.html
     */
    entry: string | 'index.html';
    /**
     * 组件包的远程入口地址，
     */
    remoteEntry?: string;
    readonly remotePackage?: string;
    readonly local?: LocalPackageUrlInfo;
    readonly remote?: RemotePackageUrlInfo;
    /**
     * 组件包图标
     */
    readonly icon?: string;
    /**
     * Hash路由模式，默认为true
     */
    readonly hash: boolean;
    /**
     * 可能是网络地址，或者本地路径（解压后的文件夹路径）,
     * 网络地址：https://www.bilibili.com
     * 本地地址：file:///C:/Users/neo/Desktop
     * 在测试时。地址通常为: http://127.0.0.1:8080
     */
    url: string;
    readonly widgets: Widget[];
    readonly pages: Page[];
    devOptions?: DevOptions;
    constructor(options: WidgetPackageOptions);
    static parseJSON(json: string): WidgetPackage;
    static parseObject(obj: any): WidgetPackage;
    /**
     * 获取组件包标题
     * @param lang 语言环境，不传则获取默认语言
     */
    getTitle(lang?: LanguageCode): string | undefined;
    /**
     * 获取组件包描述
     * @param lang 语言环境，不传则获取默认标题
     */
    getDescription(lang?: LanguageCode): string | undefined;
    /**
     * 获取组件完整路径
     * 如果url是http链接，直接返回链接
     * 如果是本地组件：file://链接，则返回 url+entry,
     * 如果是远程组件：http://链接，则返回 entry,
     * @example file://C:/users/neo/desktop/index.html#
     * @example file://C:/users/neo/desktop/index.html#
     */
    getIndexUrl(hash?: boolean): string;
}
type DevOptions = {
    folder?: string;
    route?: boolean;
    devUrl?: string;
    remoteEntry?: string;
};

declare class WidgetParams {
    static readonly PARAM_PREFIX = "w_";
    static readonly PARAM_ID = "id";
    static readonly PARAM_WIDTH = "width";
    static readonly PARAM_HEIGHT = "height";
    static readonly PARAM_WIDTH_PX = "width_px";
    static readonly PARAM_HEIGHT_PX = "height_px";
    static readonly PARAM_X = "x";
    static readonly PARAM_Y = "y";
    static readonly PARAM_LANG = "lang";
    static readonly PARAM_THEME = "theme";
    static readonly PARAM_MODE = "mode";
    static readonly PARAM_RADIUS = "radius";
    static readonly PARAM_NAME = "name";
    static readonly PARAM_TITLE = "title";
    static readonly PARAM_PREVIEW = "preview";
    static readonly PARAMS: string[];
    id?: string;
    width?: number;
    widthPx?: number;
    heightPx?: number;
    height?: number;
    x?: number;
    y?: number;
    preview?: boolean;
    lang?: string;
    theme?: ThemeMode;
    mode?: DeployMode;
    radius?: number;
    name?: string;
    title?: string;
    /**
     * 将组件参数转为url参数
     * @param object
     * @return URLSearchParams  w_w=2&w_h=2&w_id=21&w_width=156&w_height=156
     */
    toUrlParams(): URLSearchParams;
    getPersistKey(): string;
    /**
     * 从当前地址解析组件参数：
     * http://localhost:8080/#/widget/config/labor_progress?w_w=2&w_h=2&w_width=156&w_height=156
     * =>
     *  {width:2,height:2,id:21,width_px:156,height_px:156}
     */
    static fromCurrentLocation(): WidgetParams;
    static fromLocation(url: string): WidgetParams;
    private static setValue;
    /**
     * 从对象键值对中初始化组件参数
     * {w_width:2,w_height:2,w_id:21,w_width_px:156,w_height_px:156}=>
     *  {width:2,height:2,id:21,width_px:156,height_px:156}
     * @param object
     */
    static fromObject(object: any): WidgetParams;
}
declare enum ThemeMode {
    AUTO = "auto",
    LIGHT = "LIGHT",
    DARK = "DARK"
}

declare abstract class BaseApi<T extends string> {
    abstract getChannel(): string;
    protected invokeMethod(method: T, ...args: any[]): Promise<any>;
    protected invoke(...args: any[]): Promise<any>;
}

interface IWidgetApi {
    registerWidgets(widgets: Widget[]): Promise<void>;
    registerWidgetPackage(widgetPackage: WidgetPackage): Promise<void>;
    getWidgets(): Promise<Widget[]>;
    getWidget(name: string): Promise<Widget>;
    getWidgetPackage(name: string): Promise<WidgetPackage>;
    getWidgetPackages(): Promise<WidgetPackage[]>;
    getWidgetUrl(widgetName: string, widgetParams: WidgetParams): Promise<string | null>;
    getWidgetPackageIndexUrl(packageName: string, hash?: boolean): Promise<string | null>;
    getWidgetConfigUrl(widgetName: string, widgetParams: WidgetParams): Promise<string | null>;
    openConfigPage(id: string): Promise<void>;
    openConfigPageByName(name: string): Promise<void>;
    getWidgetPackageUrl(packageName: string): Promise<string | null>;
}
type WidgetApiMethods = keyof IWidgetApi;
declare enum WidgetApiEvent {
    DATA_CHANGED = "event::cn.widgetjs.core.widget.data-changed",
    EDIT_DESKTOP_WIDGETS = "event::cn.widgetjs.core.widget.desktop.edit"
}
declare class WidgetApiImpl extends BaseApi<WidgetApiMethods> implements IWidgetApi {
    getChannel(): string;
    registerWidgets(widgets: Widget[]): Promise<any>;
    registerWidgetPackage(widgetPackage: WidgetPackage): Promise<any>;
    getWidgets(): Promise<Widget[]>;
    openConfigPage(widgetId: string): Promise<any>;
    openConfigPageByName(widgetName: string): Promise<any>;
    getWidgetPackages(): Promise<WidgetPackage[]>;
    /**
     *
     * @param name package name
     */
    getWidget(name: string): Promise<Widget>;
    /**
     *
     * @param name package name
     */
    getWidgetPackage(name: string): Promise<WidgetPackage>;
    /**
     * 获取组件配置地址
     * @param widgetName
     */
    getWidgetConfigUrl(widgetName: string, widgetParams: WidgetParams): Promise<string | null>;
    getWidgetUrl(widgetName: string, widgetParams: WidgetParams): Promise<string | null>;
    getWidgetPackageIndexUrl(packageName: string, hash?: boolean): Promise<string | null>;
    getWidgetPackageUrl(packageName: string): Promise<string | null>;
}
declare const WidgetApi: WidgetApiImpl;

interface IAppApi {
    setConfig(key: string, value: string | number | boolean): Promise<void>;
    getConfig(key: string, defaultValue: string | number | boolean): Promise<string | number | boolean>;
    getVersion(): Promise<string>;
    getPreloadPath(): Promise<string>;
    openAddWidgetWindow(): Promise<void>;
    openSettingWindow(): Promise<void>;
    getIconFile(): Promise<string>;
}
type AppApiMethods = keyof IAppApi;
declare enum AppApiEvent {
    CONFIG_CHANGED = "event::cn.widgetjs.core.app.config.changed",
    MOVING_GRID_WINDOW = "event::cn.widgetjs.core.app.moving.grid.window",
    STOP_MOVING_GRID_WINDOW = "event::cn.widgetjs.core.app.moving.grid.window.stop"
}
declare enum AppApiConstants {
    CONFIG_GRID_CELL_SIZE = "cn.widgetjs.config.grid.size"
}
declare const AppApi: IAppApi;

interface BroadcastEventOptions {
    event: WidgetApiEvent | AppApiEvent | string;
    /**
     * 发送人，用于标记发送源，一般为组件名，如：com.example.widgets.countdown
     */
    sender?: string;
    /**
     * 广播事件携带的数据，只支持部分数据类型。
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Web_Workers_API/Structured_clone_algorithm
     */
    payload?: any;
}
declare class BroadcastEvent {
    readonly event: string;
    readonly sender?: string;
    payload?: any;
    constructor(options: BroadcastEventOptions);
}

declare enum WebSocketEventType {
    RESISTER_PACKAGE = "ws::cn.widgetjs.core.resister_package"
}
declare class WebSocketEvent {
    type: WebSocketEventType;
    payload: any;
    constructor(type: WebSocketEventType, payload: any);
}

/**
 * 组件配置数据，用于存储组件自定义页面所设置的数据
 */
declare class WidgetData {
    /**
     * 组件id
     */
    id?: string;
    /**
     * 组件名
     */
    name: string;
    /**
     * 背景颜色
     */
    backgroundColor?: string;
    /**
     * 文字颜色
     */
    color?: string;
    /**
     * 字体大小
     */
    fontSize?: number;
    /**
     * 字体
     */
    fontFamily?: string;
    /**
     * 圆角半径
     */
    borderRadius?: number;
    constructor(name: string, id?: string);
    parseJSON(json: {}): void;
}

type NotificationType = "countdown" | "advance-countdown" | "error" | "success" | "warning" | "info" | "reminder" | "url" | "call";
declare enum NotificationSize {
    SMALL = "small",
    NORMAL = "normal",
    LARGE = "large"
}
interface NotificationOption {
    type?: NotificationType;
    title?: string;
    message: string;
    targetTime?: string;
    duration?: number;
    /**
     * 图片名，目前只支持mingcute图标。
     * https://www.mingcute.com/
     */
    icon?: string;
    color?: string;
    confirmButtonText?: string;
    cancelButtonText?: string;
    cancelBroadcast?: string;
    confirmBroadcast?: string;
    size?: NotificationSize;
    url?: string;
    avatar?: string;
    audio?: string;
    lyric?: string;
    backgroundColor?: string;
}
declare class AppNotification {
    type: NotificationType;
    message: string;
    title?: string;
    targetTime?: string;
    duration: number;
    icon?: string;
    color?: string;
    backgroundColor?: string;
    confirmButtonText?: string;
    cancelButtonText?: string;
    cancelBroadcast?: string;
    confirmBroadcast?: string;
    size: NotificationSize;
    url?: string;
    avatar?: string;
    audio?: string;
    lyric?: string;
    createdAt: string;
    constructor(option: NotificationOption);
}

interface IDeployedPage {
    id: string;
    name: string;
    packageName: string;
    x: number;
    y: number;
    height: number;
    width: number;
    proxy?: string;
}
declare class DeployedPage implements IDeployedPage {
    name: string;
    packageName: string;
    x: number;
    y: number;
    height: number;
    width: number;
    proxy?: string;
    id: string;
}

declare class DeployedWidget extends DeployedPage {
    shortcut?: string;
    deployMode: number;
    isOverlap(): boolean;
}

interface Rectangle {
    /**
     * The height of the rectangle (must be an integer).
     */
    height: number;
    /**
     * The width of the rectangle (must be an integer).
     */
    width: number;
    /**
     * The x coordinate of the origin of the rectangle (must be an integer).
     */
    x: number;
    /**
     * The y coordinate of the origin of the rectangle (must be an integer).
     */
    y: number;
}

interface Point {
    x: number;
    y: number;
}
type Position = {
    x: number;
    y: number;
};
declare enum Gravity {
    TOP = "TOP",
    LEFT = "LEFT",
    RIGHT = "RIGHT",
    BOTTOM = "BOTTOM"
}

interface NativeKeyboardEvent {
    name: string;
    keyCode: number;
    scanCode: number;
    isKeyUp: boolean;
}

interface AppMouseEvent {
    handled: boolean;
    wheelScrolled: boolean;
    clicked: boolean;
    isMouseButtonDown: boolean;
    isMouseButtonUp: boolean;
    timestamp: number;
    button: number;
    clicks: number;
    x: number;
    y: number;
    delta: number;
    location: string;
}

declare class ElectronUtils {
    static hasElectronApi(): boolean;
    /**
     * 获取ElectronAPI
     * windows api
     */
    static getAPI(): any;
    static invokeMethod(channel: string, method: string, ...args: any[]): Promise<any>;
    static invoke(channel: string, ...args: any[]): Promise<any>;
}

declare class WidgetUrlUtils {
    static getWidgetUrl(widgetUrl: string, widgetPackage: WidgetPackage, widgetParams: WidgetParams): string;
    static getWidgetPackageUrl(url: string, entry: string, hash: boolean): string;
}

/**
 * 根据语言环境获取文本
 * @param langMap       对象
 * @param {LanguageCode} langCode  语言环境 例如：zh-CN，en-US
 * @param fallbackToBrowser
 */
declare function getTextByLanguageCode(langMap: LanguageTextMap, langCode?: LanguageCode, fallbackToBrowser?: boolean): string | undefined;

/**
 * Possible values in normalized {@link LocationQuery}. `null` renders the query
 * param but without an `=`.
 *
 * @example
 * ```
 * ?isNull&isEmpty=&other=other
 * gives
 * `{ isNull: null, isEmpty: '', other: 'other' }`.
 * ```
 *
 * @internal
 */
type LocationQueryValue = string | null;
/**
 * Possible values when defining a query.
 *
 * @internal
 */
type LocationQueryValueRaw = LocationQueryValue | number | undefined;
/**
 * Normalized query object that appears in {@link RouteLocationNormalized}
 *
 * @public
 */
type LocationQuery = Record<string, LocationQueryValue | LocationQueryValue[]>;
/**
 * Loose {@link LocationQuery} object that can be passed to functions like
 * {@link Router.push} and {@link Router.replace} or anywhere when creating a
 * {@link RouteLocationRaw}
 *
 * @public
 */
type LocationQueryRaw = Record<string | number, LocationQueryValueRaw | LocationQueryValueRaw[]>;
/**
 * Transforms a queryString into a {@link LocationQuery} object. Accept both, a
 * version with the leading `?` and without Should work as URLSearchParams
 * @internal
 *
 * @param search - search string to parse
 * @returns a query object
 */
declare function parseQuery(search: string): LocationQuery;
/**
 * Stringifies a {@link LocationQueryRaw} object. Like `URLSearchParams`, it
 * doesn't prepend a `?`
 *
 * @internal
 *
 * @param query - query object to stringify
 * @returns string version of the query without the leading `?`
 */
declare function stringifyQuery(query: LocationQueryRaw): string;

type NormalizeOptions = {
    /**
     @default 'http'
     */
    defaultProtocol?: string | 'http' | 'https';
    /**
     Prepends `defaultProtocol` to the URL if it's protocol-relative.
  
     @default true
  
     @example
     ```
     normalizeUrl('//sindresorhus.com');
     //=> 'http://sindresorhus.com'
  
     normalizeUrl('//sindresorhus.com', {normalizeProtocol: false});
     //=> '//sindresorhus.com'
     ```
     */
    readonly normalizeProtocol?: boolean;
    /**
     Normalizes HTTPS URLs to HTTP.
  
     @default false
  
     @example
     ```
     normalizeUrl('https://sindresorhus.com');
     //=> 'https://sindresorhus.com'
  
     normalizeUrl('https://sindresorhus.com', {forceHttp: true});
     //=> 'http://sindresorhus.com'
     ```
     */
    readonly forceHttp?: boolean;
    /**
     Normalizes HTTP URLs to HTTPS.
  
     This option cannot be used with the `forceHttp` option at the same time.
  
     @default false
  
     @example
     ```
     normalizeUrl('http://sindresorhus.com');
     //=> 'http://sindresorhus.com'
  
     normalizeUrl('http://sindresorhus.com', {forceHttps: true});
     //=> 'https://sindresorhus.com'
     ```
     */
    readonly forceHttps?: boolean;
    /**
     Strip the [authentication](https://en.wikipedia.org/wiki/Basic_access_authentication) part of a URL.
  
     @default true
  
     @example
     ```
     normalizeUrl('https://user:password@sindresorhus.com');
     //=> 'https://sindresorhus.com'
  
     normalizeUrl('https://user:password@sindresorhus.com', {stripAuthentication: false});
     //=> 'https://user:password@sindresorhus.com'
     ```
     */
    readonly stripAuthentication?: boolean;
    /**
     Removes hash from the URL.
  
     @default false
  
     @example
     ```
     normalizeUrl('sindresorhus.com/about.html#contact');
     //=> 'http://sindresorhus.com/about.html#contact'
  
     normalizeUrl('sindresorhus.com/about.html#contact', {stripHash: true});
     //=> 'http://sindresorhus.com/about.html'
     ```
     */
    readonly stripHash?: boolean;
    /**
     Remove the protocol from the URL: `http://sindresorhus.com` → `sindresorhus.com`.
  
     It will only remove `https://` and `http://` protocols.
  
     @default false
  
     @example
     ```
     normalizeUrl('https://sindresorhus.com');
     //=> 'https://sindresorhus.com'
  
     normalizeUrl('sindresorhus.com', {stripProtocol: true});
     //=> 'sindresorhus.com'
     ```
     */
    readonly stripProtocol?: boolean;
    /**
     Strip the [text fragment](https://web.dev/text-fragments/) part of the URL
  
     __Note:__ The text fragment will always be removed if the `stripHash` option is set to `true`, as the hash contains the text fragment.
  
     @default true
  
     @example
     ```
     normalizeUrl('http://sindresorhus.com/about.html#:~:text=hello');
     //=> 'http://sindresorhus.com/about.html#'
  
     normalizeUrl('http://sindresorhus.com/about.html#section:~:text=hello');
     //=> 'http://sindresorhus.com/about.html#section'
  
     normalizeUrl('http://sindresorhus.com/about.html#:~:text=hello', {stripTextFragment: false});
     //=> 'http://sindresorhus.com/about.html#:~:text=hello'
  
     normalizeUrl('http://sindresorhus.com/about.html#section:~:text=hello', {stripTextFragment: false});
     //=> 'http://sindresorhus.com/about.html#section:~:text=hello'
     ```
     */
    readonly stripTextFragment?: boolean;
    /**
     Removes `www.` from the URL.
  
     @default true
  
     @example
     ```
     normalizeUrl('http://www.sindresorhus.com');
     //=> 'http://sindresorhus.com'
  
     normalizeUrl('http://www.sindresorhus.com', {stripWWW: false});
     //=> 'http://www.sindresorhus.com'
     ```
     */
    readonly stripWWW?: boolean;
    /**
     Removes query parameters that matches any of the provided strings or regexes.
  
     @default [/^utm_\w+/i]
  
     @example
     ```
     normalizeUrl('www.sindresorhus.com?foo=bar&ref=test_ref', {
          removeQueryParameters: ['ref']
      });
     //=> 'http://sindresorhus.com/?foo=bar'
     ```
  
     If a boolean is provided, `true` will remove all the query parameters.
  
     ```
     normalizeUrl('www.sindresorhus.com?foo=bar', {
          removeQueryParameters: true
      });
     //=> 'http://sindresorhus.com'
     ```
  
     `false` will not remove any query parameter.
  
     ```
     normalizeUrl('www.sindresorhus.com?foo=bar&utm_medium=test&ref=test_ref', {
          removeQueryParameters: false
      });
     //=> 'http://www.sindresorhus.com/?foo=bar&ref=test_ref&utm_medium=test'
     ```
     */
    readonly removeQueryParameters?: ReadonlyArray<RegExp | string> | boolean;
    /**
     Keeps only query parameters that matches any of the provided strings or regexes.
  
     __Note__: It overrides the `removeQueryParameters` option.
  
     @default undefined
  
     @example
     ```
     normalizeUrl('https://sindresorhus.com?foo=bar&ref=unicorn', {
          keepQueryParameters: ['ref']
      });
     //=> 'https://sindresorhus.com/?ref=unicorn'
     ```
     */
    readonly keepQueryParameters?: ReadonlyArray<RegExp | string>;
    /**
     Removes trailing slash.
  
     __Note__: Trailing slash is always removed if the URL doesn't have a pathname unless the `removeSingleSlash` option is set to `false`.
  
     @default true
  
     @example
     ```
     normalizeUrl('http://sindresorhus.com/redirect/');
     //=> 'http://sindresorhus.com/redirect'
  
     normalizeUrl('http://sindresorhus.com/redirect/', {removeTrailingSlash: false});
     //=> 'http://sindresorhus.com/redirect/'
  
     normalizeUrl('http://sindresorhus.com/', {removeTrailingSlash: false});
     //=> 'http://sindresorhus.com'
     ```
     */
    readonly removeTrailingSlash?: boolean;
    /**
     Remove a sole `/` pathname in the output. This option is independent of `removeTrailingSlash`.
  
     @default true
  
     @example
     ```
     normalizeUrl('https://sindresorhus.com/');
     //=> 'https://sindresorhus.com'
  
     normalizeUrl('https://sindresorhus.com/', {removeSingleSlash: false});
     //=> 'https://sindresorhus.com/'
     ```
     */
    readonly removeSingleSlash?: boolean;
    /**
     Removes the default directory index file from path that matches any of the provided strings or regexes.
     When `true`, the regex `/^index\.[a-z]+$/` is used.
  
     @default false
  
     @example
     ```
     normalizeUrl('www.sindresorhus.com/foo/default.php', {
          removeDirectoryIndex: [/^default\.[a-z]+$/]
      });
     //=> 'http://sindresorhus.com/foo'
     ```
     */
    removeDirectoryIndex?: boolean | ReadonlyArray<RegExp | string>;
    /**
     Removes an explicit port number from the URL.
  
     Port 443 is always removed from HTTPS URLs and 80 is always removed from HTTP URLs regardless of this option.
  
     @default false
  
     @example
     ```
     normalizeUrl('sindresorhus.com:123', {
          removeExplicitPort: true
      });
     //=> 'http://sindresorhus.com'
     ```
     */
    readonly removeExplicitPort?: boolean;
    /**
     Sorts the query parameters alphabetically by key.
  
     @default true
  
     @example
     ```
     normalizeUrl('www.sindresorhus.com?b=two&a=one&c=three', {
          sortQueryParameters: false
      });
     //=> 'http://sindresorhus.com/?b=two&a=one&c=three'
     ```
     */
    readonly sortQueryParameters?: boolean;
};
declare function normalizeUrl(urlString: string, options?: NormalizeOptions): string;

interface IGridRect {
    top: number;
    left: number;
    width: number;
    height: number;
}
declare class GridRect implements IGridRect {
    top: number;
    left: number;
    width: number;
    height: number;
    constructor(left: number, top: number, width: number, height: number);
    get right(): number;
    get bottom(): number;
}
declare class GridSystem extends GridRect {
    cellSize: number;
    horizontalCellCount: number;
    verticalCellCount: number;
    screenWidth: number;
    screenHeight: number;
    /**
     *
     * @param cellSize
     * @param padding     至少保留的边距
     * @param screenWidth
     * @param screenHeight
     */
    constructor(cellSize: number, padding: number, screenWidth: number, screenHeight: number);
    /**
     * 将坐标对齐到网格上
     * @param x
     * @param y
     */
    snapPosition(x: number, y: number): number[];
    snapSize(width: number, height: number): number[];
    /**
     * 将矩形对齐到网格上
     * @param rect
     * @param minWidth
     * @param maxWidth
     * @param minHeight
     * @param maxHeight
     */
    snapRect(rect: GridRect, minWidth: number, maxWidth: number, minHeight: number, maxHeight: number): IGridRect;
    private getRangeValue;
    getWidth(): number;
    getHeight(): number;
}

declare class ElectronApi {
    static addIpcListener(key: String, f: Function): Promise<void>;
    static removeIpcListener(key: String): Promise<void>;
}

interface SaveWidgetOption {
    sendBroadcast?: boolean;
    id?: string;
}
interface IWidgetDataApi {
    findByName<T extends WidgetData>(name: string, type: {
        new (name: string, id?: string): T;
    }): Promise<T | undefined>;
    findByName<T extends WidgetData>(data: T): Promise<T | undefined>;
    save(data: WidgetData, options?: SaveWidgetOption): Promise<string>;
    getStore(name: string): LocalForage;
    saveByName<T extends WidgetData>(data: T, options?: SaveWidgetOption): Promise<string>;
    find<T extends WidgetData>(name: string, id: string, type: {
        new (name: string, id?: string): T;
    }): Promise<T | undefined>;
}
type WidgetDataApiMethods = keyof IWidgetDataApi;
declare const WidgetDataApi: IWidgetDataApi;

interface IBrowserWindowApi {
    setIgnoreMouseEvent(ignore: boolean): Promise<void>;
    show(): Promise<void>;
    hide(): Promise<void>;
    close(): Promise<void>;
    showInactive(): Promise<void>;
    center(): Promise<void>;
    startDraggingWindow(): Promise<void>;
    minimize(): Promise<void>;
    restore(): Promise<void>;
    isMinimized(): Promise<boolean>;
    isMaximized(): Promise<boolean>;
    maximize(): Promise<void>;
    stopDraggingWindow(): Promise<void>;
    setAlwaysOnTop(alwaysOnTop: boolean): Promise<void>;
    isAlwaysOnTop(): Promise<boolean>;
    openUrl(url: string, option?: OpenUrlOptions): Promise<void>;
    moveTop(): Promise<void>;
    unmaximize(): Promise<void>;
    reload(): Promise<void>;
    setSize(width: number, height: number, animate?: boolean): Promise<void>;
    openDevTools(): Promise<void>;
    setPosition(options: SetPositionOptions): Promise<void>;
    getPosition(): Promise<Position>;
    blur(): Promise<void>;
    focus(): Promise<void>;
    setResizable(resizable: boolean): Promise<void>;
    setMovable(movable: boolean): Promise<void>;
    getBounds(): Promise<Rectangle>;
    setBounds(bounds: Partial<Rectangle>, animate: boolean): Promise<void>;
    alignToScreen(align: AlignPosition): Promise<void>;
    existsByUrl(url: string): Promise<boolean>;
    getMaximumSize(): Promise<number[]>;
    getMinimumSize(): Promise<number[]>;
    setMaximumSize(width: number, height: number): Promise<void>;
    setMinimumSize(width: number, height: number): Promise<void>;
    setZoomLevel(level: number): Promise<void>;
    setZoomFactor(factor: number): Promise<void>;
    isFocused(): Promise<boolean>;
}
type BrowserWindowApiMethods = keyof IBrowserWindowApi;
type AlignPosition = 'top-left' | 'top-center' | 'top-right' | 'bottom-left' | 'bottom-center' | 'bottom-right';
interface OpenUrlOptions {
    partition?: string;
    external?: boolean;
}
interface SetPositionOptions {
    x?: number;
    y?: number;
    /**
     * 只有在 macOS 上有效
     */
    animation?: boolean;
}
declare enum BrowserWindowApiEvent {
    BLUR = "event::cn.widgetjs.core.browser-window.blur",
    MOVED = "event::cn.widgetjs.core.browser-window.moved",
    FOCUS = "event::cn.widgetjs.core.browser-window.focus",
    SNAP_TO_EDGE = "event::cn.widgetjs.core.browser-window.snap_to_edge",
    RESIZED = "event::cn.widgetjs.core.browser-window.resized",
    CANCEL_SNAP_TO_EDGE = "event::cn.widgetjs.core.browser-window.cancel_snap_to_edge"
}
interface SetupOptions {
    width: number;
    height: number;
    minWidth?: number;
    minHeight?: number;
    maxWidth?: number;
    maxHeight?: number;
    resizable?: boolean;
    movable?: boolean;
    center?: boolean;
    alwaysOnTop?: boolean;
}
declare class BrowserWindowApiImpl extends BaseApi<BrowserWindowApiMethods> implements IBrowserWindowApi {
    getChannel(): string;
    setIgnoreMouseEvent(ignore: boolean): Promise<void>;
    show(): Promise<void>;
    showInactive(): Promise<void>;
    hide(): Promise<void>;
    center(): Promise<void>;
    setAlwaysOnTop(alwaysOnTop: boolean): Promise<void>;
    isAlwaysOnTop(): Promise<boolean>;
    openUrl(url: string, option?: OpenUrlOptions): Promise<void>;
    moveTop(): Promise<void>;
    openDevTools(): Promise<void>;
    setPosition(options: SetPositionOptions): Promise<void>;
    getPosition(): Promise<Position>;
    blur(): Promise<any>;
    focus(): Promise<any>;
    /**
     * 设置窗口是否可以拉伸
     * @param resizable
     */
    setResizable(resizable: boolean): Promise<any>;
    getBounds(): Promise<Rectangle>;
    setBounds(bounds: Partial<Rectangle>, animate?: boolean): Promise<void>;
    alignToScreen(align: AlignPosition): Promise<any>;
    startDraggingWindow(): Promise<any>;
    stopDraggingWindow(): Promise<any>;
    /**
     * 通过url检测窗口是否存在
     * @param url
     */
    existsByUrl(url: string): Promise<boolean>;
    getMaximumSize(): Promise<number[]>;
    minimize(): Promise<void>;
    maximize(): Promise<void>;
    isMaximized(): Promise<boolean>;
    isMinimized(): Promise<boolean>;
    restore(): Promise<void>;
    unmaximize(): Promise<void>;
    setZoomLevel(level: number): Promise<void>;
    reload(): Promise<void>;
    setMovable(movable: boolean): Promise<void>;
    setSize(width: number, height: number, animate?: boolean): Promise<void>;
    isFocused(): Promise<boolean>;
    setMaximumSize(width: number, height: number): Promise<void>;
    setMinimumSize(width: number, height: number): Promise<void>;
    getMinimumSize(): Promise<number[]>;
    close(): Promise<void>;
    setZoomFactor(factor: number): Promise<void>;
    setup(options: SetupOptions): Promise<void>;
}
declare const BrowserWindowApi: BrowserWindowApiImpl;

interface INotificationApi {
    send(notification: AppNotification): Promise<void>;
    hide(): Promise<void>;
}
type NotificationApiMethods = keyof INotificationApi;
declare enum NotificationApiEvent {
    CONFIRM = "event::cn.widgetjs.core.notification.confirm",
    CANCEL = "event::cn.widgetjs.core.notification.cancel",
    HIDE = "event::cn.widgetjs.core.notification.hide"
}
declare class NotificationApiImpl extends BaseApi<NotificationApiMethods> implements INotificationApi {
    /**
     * 来电
     * @param avatar    头像地址
     * @param audio     音频地址
     * @param title     标题文件
     * @param message   初始消息
     * @param lyric     歌词字符串
     */
    call(avatar: string, audio: string, title: string, message: string, lyric: string): Promise<void>;
    send(notification: AppNotification): Promise<any>;
    reminder(title: string, message: string, icon: string, cancelButtonText: string, confirmButtonText: string, cancelBroadcast: string, confirmBroadcast: string, duration?: number): Promise<any>;
    advanceCountdown(message: string, targetTime: string, title?: string): Promise<any>;
    countdown(message: string, targetTime: string): Promise<void>;
    success(message: string, duration?: number): Promise<void>;
    error(message: string, duration?: number): Promise<void>;
    warning(message: string, duration?: number): Promise<void>;
    info(message: string, duration?: number): Promise<void>;
    /**
     * 隐藏通知
     */
    hide(): Promise<void>;
    getChannel(): string;
}
declare const NotificationApi: NotificationApiImpl;

interface IDeployedWidgetApi {
    removeDeployedWidget(id: string): Promise<void>;
    removeDeployedWidgetByName(name: string): Promise<DeployedWidget[]>;
    getDeployedWidgets(): Promise<DeployedWidget[]>;
    openDevTools(id: string): Promise<void>;
    openConfigPage(id: string): Promise<void>;
    registerActiveShortcut(id: string, shortcut: string): Promise<boolean>;
    setProxy(id: string, proxy: string): Promise<boolean>;
    getDeployedWidget(id: string): Promise<DeployedWidget>;
    addWidget(widgetName: string, hostMode: DeployMode): Promise<Widget[]>;
}
type DeployedWidgetApiMethods = keyof IDeployedWidgetApi;
declare class DeployedWidgetApiImpl extends BaseApi<DeployedWidgetApiMethods> implements IDeployedWidgetApi {
    getChannel(): string;
    /**
     * 移除组件
     * @param id
     */
    removeDeployedWidget(id: string): Promise<any>;
    addWidget(widgetName: string, hostMode: DeployMode): Promise<Widget[]>;
    /**
     * 通过组件名移除已添加的组件
     * @param name  组件名
     */
    removeDeployedWidgetByName(name: string): Promise<DeployedWidget[]>;
    /**
     * 获取已添加的组件
     * @param name  组件名，可以不传
     */
    getDeployedWidgets(name?: string): Promise<DeployedWidget[]>;
    getDeployedWidget(id: string): Promise<DeployedWidget>;
    /**
     * Opens the dev tools for a widget with the specified ID.
     *
     * @param {string} widgetId - The ID of the widget to open the dev tools for.
     * @return {Promise} A Promise that resolves when the dev tools are opened.
     */
    openDevTools(widgetId: string): Promise<any>;
    openConfigPage(widgetId: string, params?: WidgetParams): Promise<any>;
    /**
     * 注册激活、呼出、置顶组件快捷键
     * @param widgetId  组件id
     * @param shortcut  如果传空或者不传，则会取消快捷键。更多快捷键配置，请查看Accelerator用法
     *                  https://www.electronjs.org/docs/latest/api/accelerator
     */
    registerActiveShortcut(widgetId: string, shortcut?: string): Promise<boolean>;
    setProxy(widgetId: string, proxy: string): Promise<boolean>;
}
declare const DeployedWidgetApi: DeployedWidgetApiImpl;

interface IDeviceApi {
    getCursorScreenPoint(): Promise<Point>;
    isAllKeyReleased(): Promise<boolean>;
}
type DeviceApiMethods = keyof IDeviceApi;
declare const DeviceApi: IDeviceApi;

declare enum Channel {
    NOTIFICATION = "channel::cn.widgetjs.core.notification",
    BROWSER_WINDOW = "channel::cn.widgetjs.core.browser_window",
    BROADCAST = "channel::cn.widgetjs.core.broadcast",
    WIDGET = "channel::cn.widgetjs.core.widget",
    HOSTED_WIDGET = "channel::cn.widgetjs.core.hosted_widget",
    APP = "channel::cn.widgetjs.core.app",
    DIALOG = "channel::cn.widgetjs.core.dialog",
    CLIPBOARD = "channel::cn.widgetjs.core.clipboard",
    FILE = "channel::cn.widgetjs.core.file",
    SCREEN = "channel::cn.widgetjs.core.screen",
    SHORTCUT = "channel::cn.widgetjs.core.shortcut",
    LOG = "channel::cn.widgetjs.core.log",
    DEVICE = "channel::cn.widgetjs.core.device",
    MOUSE = "channel::cn.widgetjs.core.mouse",
    KEYBOARD = "channel::cn.widgetjs.core.keyboard",
    STORE = "channel::cn.widgetjs.core.store"
}

interface IClipboardApi {
    getSelectedText(): Promise<string | undefined>;
    getText(): Promise<string | undefined>;
}
type ClipboardApiMethods = keyof IClipboardApi;
declare enum ClipboardApiEvent {
    CHANGED = "clipboard-changed"
}
declare const ClipboardApi: IClipboardApi;

declare class ApiConstants {
    static readonly CONFIG_LAUNCH_AT_STARTUP = "CONFIG_LAUNCH_AT_STARTUP";
    static readonly CONFIG_WIDGET_TITLE_COLOR = "CONFIG_WIDGET_TITLE_COLOR";
    static readonly CONFIG_DEBUG_MODE = "cn.widgetjs.config.debug";
    static readonly CONFIG_GRID_CELL_SIZE = "cn.widgetjs.config.grid.size";
    static readonly SHORTCUT_PIN_DESKTOP_WIDGETS = "cn.widgetjs.config.shortcut.pin_desktop_widgets";
}

interface IShortcutApi {
    register(shortcut: string): void;
    unregister(shortcut: string): void;
}
type ShortcutApiMethods = keyof IShortcutApi;
declare enum ShortcutApiEvent {
    TRIGGERED = "channel::cn.widgetjs.core.shortcut.triggered"
}
declare const ShortcutApi: IShortcutApi;

type BroadcastEventType = WidgetApiEvent | BrowserWindowApiEvent | ShortcutApiEvent | ClipboardApiEvent | string;
interface IBroadcastApi {
    send(event: BroadcastEvent): Promise<void>;
    register(...event: BroadcastEventType[]): Promise<void>;
    unregister(...event: BroadcastEventType[]): Promise<void>;
}
type BroadcastApiMethods = keyof IBroadcastApi;
declare const BroadcastApi: IBroadcastApi;

interface IDialogApi {
    pickFile(extensions?: string[]): Promise<string | undefined>;
    pickFolder(): Promise<string | undefined>;
}
type DialogApiMethods = keyof IDialogApi;
declare const DialogApi: IDialogApi;

type BaseType = string | number | boolean;
interface IStoreApi {
    get<T extends BaseType>(key: string, defaultValue?: T): Promise<T | null>;
    set(key: string, value: BaseType | object): Promise<string>;
    getObject<T>(key: string, defaultValue?: T): Promise<T | undefined>;
    delete(key: string): Promise<void>;
}
type StoreApiMethods = keyof IStoreApi;
declare const StoreApi: IStoreApi;

interface ILogApi {
    info(...data: any[]): void;
    error(...data: any[]): void;
    warn(...data: any[]): void;
    log(...data: any[]): void;
    json(data: any): void;
}
type LogApiMethods = keyof ILogApi;
declare const LogApi: ILogApi;

interface ReadDirOptions {
    ignoreDir?: boolean;
    traverseDir?: boolean;
}
interface IFileApi {
    readDirectory(path: string, options?: ReadDirOptions): Promise<string[]>;
    isDirectory(filePath: string): Promise<boolean>;
    getFileType(filePath: string): Promise<FileTypeResult>;
}
type FileApiMethods = keyof IFileApi;
declare const FileApi: IFileApi;

export { AlignPosition, ApiConstants, AppApi, AppApiConstants, AppApiEvent, AppApiMethods, AppMouseEvent, AppNotification, BaseApi, BroadcastApi, BroadcastApiMethods, BroadcastEvent, BroadcastEventType, BrowserWindowApi, BrowserWindowApiEvent, BrowserWindowApiMethods, Channel, ClipboardApi, ClipboardApiEvent, ClipboardApiMethods, DeployMode, DeployedPage, DeployedWidget, DeployedWidgetApi, DeployedWidgetApiMethods, DevOptions, DeviceApi, DeviceApiMethods, DialogApi, DialogApiMethods, ElectronApi, ElectronUtils, FileApi, FileApiMethods, Gravity, GridRect, GridSystem, HostedMode, IGridRect, IPage, IPageOptions, IWidgetDataApi, IWidgetOptions, IWindowSize, LanguageCode, LanguageTextMap, LocalPackageUrlInfo, LocationQuery, LocationQueryRaw, LocationQueryValue, LocationQueryValueRaw, LogApi, LogApiMethods, MetaInfo, NativeKeyboardEvent, NormalizeOptions, NotificationApi, NotificationApiEvent, NotificationApiMethods, NotificationOption, NotificationSize, NotificationType, OpenUrlOptions, Page, Point, Position, ReadDirOptions, Rectangle, RemotePackageUrlInfo, SaveWidgetOption, SetPositionOptions, ShortcutApi, ShortcutApiEvent, ShortcutApiMethods, StoreApi, StoreApiMethods, ThemeMode, WebSocketEvent, WebSocketEventType, Widget, WidgetApi, WidgetApiEvent, WidgetApiMethods, WidgetData, WidgetDataApi, WidgetDataApiMethods, WidgetKeyword, WidgetPackage, WidgetPackageOptions, WidgetParams, WidgetPermission, WidgetRoute, WidgetUrlUtils, getTextByLanguageCode, normalizeUrl, parseQuery, stringifyQuery };
